@@ -6,16 +6,17 @@ use strict;
 use 5.010000;
 no warnings 'utf8';
 
-our $VERSION = '0.044';
+our $VERSION = '0.045';
 
 use File::Temp qw( tempfile );
 
 use Clone                  qw( clone );
 use File::Slurp            qw( read_file );
+use List::Util             qw( all );
 use Term::Choose           qw();
 use Term::Choose::Util     qw( choose_multi );
 use Term::ReadLine::Simple qw();
-use Text::ParseWords       qw( parse_line );
+use Text::CSV              qw();
 
 use App::DBBrowser::Util;
 
@@ -37,7 +38,7 @@ sub __insert_into {
     $sql->{print}{chosen_cols}      = [];
     my $sql_type = 'Insert';
 
-    COLUMNS: while ( 1 ) {
+    COL_NAMES: while ( 1 ) {
         my @pre = ( $self->{info}{ok} );
         unshift @pre, undef if $self->{opt}{sssc_mode};
         my $choices = [ @pre, @cols ];
@@ -60,7 +61,7 @@ sub __insert_into {
                 $sql->{quote}{chosen_cols} = [];
                 $sql->{print}{chosen_cols} = [];
                 @cols = ( @$pr_columns );
-                next COLUMNS;
+                next COL_NAMES;
             }
             else {
                 $sql = clone( $backup_sql );
@@ -77,7 +78,7 @@ sub __insert_into {
                 @{$sql->{quote}{chosen_cols}} = @{$qt_columns}{@$pr_columns};
                 @{$sql->{print}{chosen_cols}} = @$pr_columns;
             }
-            last COLUMNS;
+            last COL_NAMES;
         }
         for my $print_col ( @print_col ) {
             push @{$sql->{quote}{chosen_cols}}, $qt_columns->{$print_col};
@@ -87,7 +88,7 @@ sub __insert_into {
     my $trs = Term::ReadLine::Simple->new();
     my $insert_mode;
 
-    INSERT: while ( 1 ) {
+    VALUES: while ( 1 ) {
         if ( ! $self->{opt}{insert_mode} ) {
             $util->__print_sql_statement( $sql, $table, $sql_type );
             my $choices = [ undef, 'Cols', 'Rows', 'Multirow', 'File' ];
@@ -105,61 +106,74 @@ sub __insert_into {
             $insert_mode = $self->{opt}{insert_mode};
         }
         if ( $insert_mode <= 2 ) {
-            my ( $last, $add ) = ( 'Last', 'Add' );
+            my ( $last, $add, $del ) = ( '-OK-', 'Add', 'Del' );
             ROWS: while ( 1 ) {
-                my $row_idx = @{$sql->{quote}{insert_into_args}};
                 if ( $insert_mode == 1 ) {
-                    COLS: for my $col ( @{$sql->{print}{chosen_cols}} ) {
+                    my $row_idx = @{$sql->{quote}{insert_into_args}};
+                    COLS: for my $col_name ( @{$sql->{print}{chosen_cols}} ) {
                         $util->__print_sql_statement( $sql, $table, $sql_type );
                         # Readline
-                        my $value = $trs->readline( $col . ': ' );
-                        if ( ! defined $value ) {
-                            if ( $row_idx > 0 ) {
-                                $#{$sql->{quote}{insert_into_args}}--;
-                            }
-                            else {
-                                next INSERT if ! defined $sql->{quote}{insert_into_args}[0][0];
-                                $sql->{quote}{insert_into_args} = [];
-                            }
-                            last COLS;
-                        }
-                        push @{$sql->{quote}{insert_into_args}->[$row_idx]}, $value;
+                        my $col = $trs->readline( $col_name . ': ' );
+                        push @{$sql->{quote}{insert_into_args}->[$row_idx]}, $col; # show $col immediately in "print_sql_statement"
                     }
                 }
                 elsif ( $insert_mode == 2 ) {
+                    my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
                     $util->__print_sql_statement( $sql, $table, $sql_type );
                     # Readline
                     my $row = $trs->readline( 'Row: ' );
                     if ( ! defined $row ) {
-                            if ( $row_idx > 0 ) {
-                                $#{$sql->{quote}{insert_into_args}}--;
-                            }
-                            else {
-                                $sql->{quote}{insert_into_args} = [];
-                            }
-                            next ROWS;
+                        if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                            $sql->{quote}{chosen_cols} = [];
+                            $sql->{print}{chosen_cols} = [];
+                            return;
+                        }
+                        $#{$sql->{quote}{insert_into_args}}--;
+                        next ROWS;
                     }
-                    push @{$sql->{quote}{insert_into_args}}, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
+                    my $status = $csv->parse( $row );
+                    push @{$sql->{quote}{insert_into_args}}, [ $csv->fields() ];
                 }
-                $util->__print_sql_statement( $sql, $table, $sql_type );
-                my $choices = [ $last, $add ];
+                my $choices = [ $last, $add, $del ];
                 unshift @$choices, undef if $self->{opt}{sssc_mode};
-                # Choose
-                my $add_row = $stmt_h->choose(
-                    $choices,
-                    { prompt => '' }
-                );
-                if ( ! defined $add_row ) {
-                    $sql->{quote}{insert_into_args} = [];
-                    next INSERT;
-                }
-                elsif ( $add_row eq $last ) {
-                    return;
+                my $default = ( all { ! length } @{$sql->{quote}{insert_into_args}[-1]} ) ? 2 : 1;
+
+                ASK: while ( 1 ) {
+                    $util->__print_sql_statement( $sql, $table, $sql_type );
+                    # Choose
+                    my $add_row = $stmt_h->choose(
+                        $choices,
+                        { prompt => '', default => $default }
+                    );
+                    if ( ! defined $add_row ) {
+                        $sql->{quote}{insert_into_args} = [];
+                        $sql->{quote}{chosen_cols}      = [];
+                        $sql->{print}{chosen_cols}      = [];
+                        return;
+                    }
+                    elsif ( $add_row eq $last ) {
+                        if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                            $sql->{quote}{chosen_cols} = [];
+                            $sql->{print}{chosen_cols} = [];
+                        }
+                        return;
+                    }
+                    elsif ( $add_row eq $del ) {
+                        if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                            $sql->{quote}{chosen_cols} = [];
+                            $sql->{print}{chosen_cols} = [];
+                            return;
+                        }
+                        $default = 0;
+                        $#{$sql->{quote}{insert_into_args}}--; ###
+                        next ASK;
+                    }
+                    last ASK;
                 }
             }
         }
         else {
-            my $row_idx = @{$sql->{quote}{insert_into_args}};
+            my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
             my $fh;
             $util->__print_sql_statement( $sql, $table, $sql_type );
             if ( $insert_mode == 3 ) {
@@ -167,27 +181,25 @@ sub __insert_into {
                 # STDIN
                 my $input = read_file( \*STDIN );
                 ( $fh ) = tempfile( DIR => $self->{info}{app_dir}, UNLINK => 1 );
-                binmode $fh, ':encoding(' . $self->{opt}{encoding_in_file} . ')';
+                binmode $fh, ':encoding(' . $self->{opt}{encoding_csv_file} . ')';
                 print $fh $input;
                 seek $fh, 0, 0;
-                #for my $row ( split $/, $input ) {
-                #    push @{$sql->{quote}{insert_into_args}}, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
-                #}
+                #$sql->{quote}{insert_into_args} = $csv->getline_all( \*STDIN );
             }
             elsif ( $insert_mode == 4 ) {
                 # Readline
                 my $file = $trs->readline( 'Path to file: ' );
                 return if ! defined $file;
-                open $fh, '<:encoding(' . $self->{opt}{encoding_in_file} . ')', $file or die $!;
-                #while ( my $row = <$fh> ) {
-                #    chomp $row;
-                #    push @{$sql->{quote}{insert_into_args}}, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
-                #}
+                open $fh, '<:encoding(' . $self->{opt}{encoding_csv_file} . ')', $file or die $!;
+                #open my $fh, '<:encoding(' . $self->{opt}{encoding_csv_file} . ')', $file or die $!;
+                #$sql->{quote}{insert_into_args} = $csv->getline_all( $fh );
                 #close $fh;
             }
-            while ( my $row = <$fh> ) {
-                chomp $row;
-                push @{$sql->{quote}{insert_into_args}}, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
+            $sql->{quote}{insert_into_args} = $csv->getline_all( $fh );
+            if ( ! @{$sql->{quote}{insert_into_args}} ) {
+                $sql->{quote}{chosen_cols} = [];
+                $sql->{print}{chosen_cols} = [];
+                return;
             }
             if ( $self->{opt}{row_col_filter} ) {
                 $self->__filter_input( $sql, $table, $sql_type, $fh );
@@ -203,6 +215,7 @@ sub __filter_input {
     my ( $self, $sql, $table, $sql_type, $fh ) = @_;
     my $util = App::DBBrowser::Util->new( $self->{info}, $self->{opt} );
     my $stmt_h = Term::Choose->new( $self->{info}{lyt_stmt_h} );
+    my $csv = Text::CSV->new( { map { $_ => $self->{opt}{$_} } @{$self->{info}{csv_opt}} } );
     #my $backup = clone $sql->{quote}{insert_into_args};
 
     FILTER: while ( 1 ) {
@@ -217,21 +230,14 @@ sub __filter_input {
             { prompt => 'Filter:' }
         );
         if ( ! defined $choice ) {
-            seek $fh, 0, 0;
-            while ( my $row = <$fh> ) {
-                chomp $row;
-                push @{$sql->{quote}{insert_into_args}}, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
-            }
-            #$sql->{quote}{insert_into_args} = clone $backup;
+            $sql->{quote}{insert_into_args} = [];
+            $sql->{quote}{chosen_cols}      = [];
+            $sql->{print}{chosen_cols}      = [];
             return;
         }
         elsif ( $choice eq $reset ) {
             seek $fh, 0, 0;
-            $sql->{quote}{insert_into_args} = [];
-            while ( my $row = <$fh> ) {
-                chomp $row;
-                push @{$sql->{quote}{insert_into_args}}, [ parse_line( $self->{opt}{delim}, $self->{opt}{keep}, $row ) ];
-            }
+            $sql->{quote}{insert_into_args} = $csv->getline_all( $fh );
             #$sql->{quote}{insert_into_args} = clone $backup;
         }
         elsif ( $choice eq $self->{info}{ok} ) {
@@ -244,9 +250,9 @@ sub __filter_input {
                 my @pre = ( $self->{info}{ok} );
                 unshift @pre, undef if $self->{opt}{sssc_mode};
                 my $choices = [ @pre, map { "col_$_" } 0 .. $#{$sql->{quote}{insert_into_args}[0]} ];
-                $util->__print_sql_statement( $sql, $table, $sql_type );
                 my $prompt = 'Cols: ';
                 $prompt .= join ',', @col_idx if @col_idx;
+                $util->__print_sql_statement( $sql, $table, $sql_type );
                 # Choose
                 my @col_nr = $stmt_h->choose(
                     $choices,
@@ -306,15 +312,14 @@ sub __filter_input {
             $last_row -= @pre;
             next FILTER if $last_row < 0;
             if ( $last_row < $first_row ) {
+                $util->__print_sql_statement( $sql, $table, $sql_type );
+                # Choose
                 $stmt_h->choose(
                     [ "Last row ($last_row) is less than First row ($first_row)" ],
                     { %{$self->{info}{lyt_stop}}, prompt => '' }
                 );
                 next FILTER;
             }
-            $util->__print_sql_statement( $sql, $table, $sql_type );
-            my $prompt = sprintf "First row: %*d\n", length $last_row, $first_row;
-            $prompt .= sprintf "Last  row: %*d\n\n", length $last_row, $last_row;
             $sql->{quote}{insert_into_args} = [ @{$sql->{quote}{insert_into_args}}[$first_row .. $last_row] ];
             next FILTER;
         }
